from ..WeatherAntecedentDTO import PressureAntecedent


class PressureAntecedentFunction(object):
    def __init__(self, redis):
        self.r = redis

    def get_pressure_antecedent(self, user_id, rule_id, device_id):
        dto = PressureAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        dto.check_pressure = self.r.get(key_pattern + ":check_pressure")
        dto.pressure_condition = self.r.get(key_pattern + ":pressure_condition")
        dto.pressure_start_value = self.r.get(key_pattern + ":pressure_start_value")
        dto.pressure_stop_value = self.r.get(key_pattern + ":pressure_stop_value")
        dto.pressure_evaluation = self.r.get(key_pattern + ":pressure_evaluation")
        location_name = self.r.get("user:" + user_id + ":location:name")
        dto.pressure_value = self.r.get("weather:" + location_name + ":pressure")
        return dto

    def set_pressure_antecedent(self, user_id, rule_id, device_id, dto: PressureAntecedent):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_pressure", dto.check_pressure)
        self.r.set(key_pattern + ":pressure_condition", dto.pressure_condition)
        self.r.set(key_pattern + ":pressure_start_value", dto.pressure_start_value)
        self.r.set(key_pattern + ":pressure_stop_value", dto.pressure_stop_value)
        self.r.set(key_pattern + ":pressure_evaluation", dto.pressure_evaluation)

    def delete_pressure_antecedent(self, user_id, rule_id, device_id):
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.delete(key_pattern + ":check_pressure")
        self.r.delete(key_pattern + ":pressure_condition")
        self.r.delete(key_pattern + ":pressure_start_value")
        self.r.delete(key_pattern + ":pressure_stop_value")
        self.r.delete(key_pattern + ":pressure_evaluation")

    def register_pressure_antecedent(self, user_id, rule_id, device_id):
        dto = PressureAntecedent()
        key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
        self.r.set(key_pattern + ":check_pressure", dto.check_pressure)
        self.r.set(key_pattern + ":pressure_condition", dto.pressure_condition)
        self.r.set(key_pattern + ":pressure_start_value", dto.pressure_start_value)
        self.r.set(key_pattern + ":pressure_stop_value", dto.pressure_stop_value)
        self.r.set(key_pattern + ":pressure_evaluation", dto.pressure_evaluation)

    def pressure_antecedent_evaluation(self, user_id, rule_id, device_id):
        try:
            dto = self.get_pressure_antecedent(user_id, rule_id, device_id)
            evaluation = "true"
            if dto.check_pressure == "true":
                evaluation = "false"
                if dto.pressure_condition == "between":
                    if dto.pressure_start_value < dto.pressure_value <= dto.pressure_stop_value:
                        evaluation = "true"
                elif dto.pressure_condition == ">":
                    if dto.pressure_value > dto.pressure_start_value:
                        evaluation = "true"
                elif dto.pressure_condition == "<":
                    if dto.pressure_value < dto.pressure_stop_value:
                        evaluation = "true"
                if dto.pressure_evaluation != evaluation:
                    key_pattern = "user:" + user_id + ":rule:" + rule_id + ":rule_antecedents:" + device_id
                    self.r.set(key_pattern + ":pressure_evaluation", evaluation)
            return evaluation
        except Exception as error:
            print(repr(error))
            return "false"



from .AlertDTO import Alert
from ..DeviceId import ALERT


class AlertFunction(object):
    def __init__(self, redis):
        self.r = redis

    def register(self, user_id, email):
        try:
            device_id = ALERT + "-" + user_id
            key_pattern = "device:" + device_id
            if self.r.exists(key_pattern + ":name") == 0:
                self.r.set(key_pattern + ":name", "alert")
                self.r.set(key_pattern + ":user_id", user_id)
                self.r.rpush(key_pattern + ":email_list", email)
                return "true"
            else:
                return "false"
        except Exception as error:
            print(repr(error))
            return "error"

    def get_device(self, user_id, device_id):
        try:
            key_pattern = "device:" + device_id
            dto = Alert()
            dto.id = device_id
            dto.name = self.r.get(key_pattern + ":name")
            dto.email_list = self.r.lrange(key_pattern + ":email_list")
            if self.r.exists(key_pattern + ":rules") == 1:
                rules_id = self.r.lrange(key_pattern + ":rules")
                for rule_id in rules_id:
                    rule_name = self.r.get("user:" + user_id + ":rule:" + rule_id + ":name")
                    dto.rules.append({"id": rule_id, "name": rule_name})
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_all_email(self, device_id):
        key_pattern = "device:" + device_id
        if self.r.exists(key_pattern + ":email_list"):
            email_list = self.r.lrange(key_pattern + ":email_list")
            if len(email_list) > 0:
                for email in email_list:
                    self.r.lrem(key_pattern + ":email_list", email)

    def update_device(self, new_device):
        try:
            dto = Alert()
            dto.device_mapping(new_device)
            key_pattern = "device:" + dto.id
            self.r.set(key_pattern + ":name", dto.name)
            self.delete_all_email(dto.id)
            if len(dto.email_list) > 0:
                for email in dto.email_list:
                    self.r.rpush(key_pattern + ":email_list", email)
            return dto
        except Exception as error:
            print(repr(error))
            return "error"

    def add_alert_email(self, user_id):
        try:
            alert_id = "alert-" + user_id
            self.r.rpush("device:" + alert_id + ":email_list", "")
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def delete_alert_email(self, user_id, index):
        try:
            alert_id = "alert-" + user_id
            email_list = self.r.lrange("device:" + alert_id + ":email_list")
            email = email_list[index]
            self.r.lrem("device:" + alert_id + ":email_list", email)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"

    def modify_alert_email(self, user_id, email, idx):
        try:
            alert_id = "alert-" + user_id
            self.r.lset("device:" + alert_id + ":email_list", idx, email)
            return "true"
        except Exception as error:
            print(repr(error))
            return "error"



class Photocell(object):
    def __init__(self):
        self.id = ""
        self.name = "PHOTOCELL"
        self.measure = "-"
        self.rules = []
        self.status = "disconnected"
        self.color = "red"
        self.unit_measure = "%"
        self.max_measure = "-"
        self.max_measure_time = "-"
        self.max_measure_date = "-"
        self.min_measure = "-"
        self.min_measure_time = "-"
        self.min_measure_date = "-"
        self.expiration = "10"

    def device_mapping(self, device):
        self.id = device["id"]
        self.name = device["name"]
        self.rules = device["rules"]
        self.expiration = device["expiration"]



class Button(object):
    def __init__(self):
        self.id = ""
        self.name = "BUTTON"
        self.measure = "-"
        self.rules = []
        self.status = "disconnected"
        self.color = "red"
        self.last_time_on = "-"
        self.last_time_off = "-"
        self.last_date_on = "-"
        self.last_date_off = "-"
        self.expiration = "10"

    def device_mapping(self, device):
        self.id = device["id"]
        self.name = device["name"]
        self.rules = device["rules"]
        self.expiration = device["expiration"]



import setuptools

setuptools.setup(
    name="ruleapp",
    version="0.11.3",
    author="Giovanni Di Martino",
    author_email="giovanni.dimartino1994@gmail.com",
    description="ruleapp dto library",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages()
)


class Servo(object):
    def __init__(self):
        self.id = ""
        self.name = "servo"
        self.measure = "-"
        self.absolute_measure = "90"
        self.rules = []
        self.automatic = "true"
        self.manual_measure = "off"
        self.setting_on = "0"
        self.setting_off = "90"
        self.setting_unit_measure = "degree"
        self.last_date_on = "-"
        self.last_date_off = "-"
        self.last_time_on = "-"
        self.last_time_off = "-"
        self.status = "disconnected"
        self.color = "red"
        self.expiration = "10"

    def device_mapping(self, device):
        self.id = device["id"]
        self.name = device["name"]
        self.rules = device["rules"]
        self.automatic = device["automatic"]
        self.manual_measure = device["manual_measure"]
        self.setting_on = device["setting_on"]
        self.setting_off = device["setting_off"]
        self.expiration = device["expiration"]



class WaterLevelAntecedent(object):
    def __init__(self):
        self.device_id = ""
        self.device_name = ""
        self.measure = "-"
        self.condition_measure = "between"
        self.start_value = "//"
        self.stop_value = "//"
        self.last_time_on = "-"
        self.last_time_off = "-"
        self.last_date_on = "-"
        self.last_date_off = "-"
        self.evaluation = "false"

    def antecedent_mapping(self, antecedent):
        self.device_id = antecedent["device_id"]
        self.device_name = antecedent["device_name"]
        self.condition_measure = antecedent["condition_measure"]
        self.start_value = antecedent["start_value"]
        self.stop_value = antecedent["stop_value"]
